<?php



























class DB_common extends PEAR
{
 
 






 var $features;

 



 var $errorcode_map;

 



 var $type;

 


 var $prepare_tokens;

 


 var $prepare_types;

 


 var $prepared_queries;

 


 var $prepare_maxstmt = 0;

 


 var $last_query = '';

 


 var $fetchmode = DB_FETCHMODE_ORDERED;

 


 var $fetchmode_object_class = 'stdClass';

 





 var $options = array(
 'persistent' => false,
 'optimize' => 'performance',
 'debug' => 0,
 'seqname_format' => '%s_seq',
 'autofree' => false
 );

 



 var $dbh;

 
 
 





 function toString()
 {
 $info = get_class($this);
 $info .= ": (phptype=" . $this->phptype .
 ", dbsyntax=" . $this->dbsyntax .
 ")";

 if ($this->connection) {
 $info .= " [connected]";
 }

 return $info;
 }

 
 
 


 function DB_common()
 {
 $this->PEAR('DB_Error');
 $this->features = array();
 $this->errorcode_map = array();
 $this->fetchmode = DB_FETCHMODE_ORDERED;
 }

 
 

 







 function quoteString($string)
 {
 $string = $this->quote($string);
 if ($string{0} == "'") {
 return substr($string, 1, -1);
 }
 return $string;
 }

 









 function quote($string)
 {
 return ($string === null) ? 'NULL' : "'".str_replace("'", "''", $string)."'";
 }

 
 

 








 function provides($feature)
 {
 return $this->features[$feature];
 }

 
 

 













 function errorCode($nativecode)
 {
 if (isset($this->errorcode_map[$nativecode])) {
 return $this->errorcode_map[$nativecode];
 }
 
 return DB_ERROR;
 }

 
 

 











 function errorMessage($dbcode)
 {
 return DB::errorMessage($this->errorcode_map[$dbcode]);
 }

 
 

 




























 function &raiseError($code = DB_ERROR, $mode = null, $options = null,
 $userinfo = null, $nativecode = null)
 {
 
 if (is_object($code)) {
 
 
 if ($mode === null && !empty($this->_default_error_mode)) {
 $mode = $this->_default_error_mode;
 $options = $this->_default_error_options;
 }
 return PEAR::raiseError($code, null, $mode, $options, null, null, true);
 }

 if ($userinfo === null) {
 $userinfo = $this->last_query;
 }

 if ($nativecode) {
 $userinfo .= " [nativecode=$nativecode]";
 }

 return PEAR::raiseError(null, $code, $mode, $options, $userinfo,
 'DB_Error', true);
 }

 
 

 























 function setFetchMode($fetchmode, $object_class = null)
 {
 switch ($fetchmode) {
 case DB_FETCHMODE_OBJECT:
 if ($object_class) {
 $this->fetchmode_object_class = $object_class;
 }
 case DB_FETCHMODE_ORDERED:
 case DB_FETCHMODE_ASSOC:
 $this->fetchmode = $fetchmode;
 break;
 default:
 return $this->raiseError('invalid fetchmode mode');
 }
 }

 
 
 







 function setOption($option, $value)
 {
 if (isset($this->options[$option])) {
 $this->options[$option] = $value;
 return DB_OK;
 }
 return $this->raiseError("unknown option $option");
 }

 
 
 






 function getOption($option)
 {
 if (isset($this->options[$option])) {
 return $this->options[$option];
 }
 return $this->raiseError("unknown option $option");
 }

 
 

 



















 function prepare($query)
 {
 $tokens = split("[\&\?\!]", $query);
 $token = 0;
 $types = array();

 for ($i = 0; $i < strlen($query); $i++) {
 switch ($query[$i]) {
 case '?':
 $types[$token++] = DB_PARAM_SCALAR;
 break;
 case '&':
 $types[$token++] = DB_PARAM_OPAQUE;
 break;
 case '!':
 $types[$token++] = DB_PARAM_MISC;
 break;
 }
 }

 $this->prepare_tokens[] = &$tokens;
 end($this->prepare_tokens);

 $k = key($this->prepare_tokens);
 $this->prepare_types[$k] = $types;
 $this->prepared_queries[$k] = &$query;

 return $k;
 }

 
 

 










 function autoPrepare($table, $table_fields, $mode = DB_AUTOQUERY_INSERT, $where = false)
 {
 $query = $this->buildManipSQL($table, $table_fields, $mode, $where);
 return $this->prepare($query);
 }

 
 

 











 function autoExecute($table, $fields_values, $mode = DB_AUTOQUERY_INSERT, $where = false)
 {
 $sth = $this->autoPrepare($table, array_keys($fields_values), $mode, $where);
 return $this->execute($sth, array_values($fields_values));

 }

 
 

 















 function buildManipSQL($table, $table_fields, $mode, $where = false)
 {
 if (count($table_fields)==0) {
 $this->raiseError(DB_ERROR_NEED_MORE_DATA);
 }
 $first = true;
 switch($mode) {
 case DB_AUTOQUERY_INSERT:
 $values = '';
 $names = '';
 while (list(, $value) = each($table_fields)) {
 if ($first) {
 $first = false;
 } else {
 $names .= ',';
 $values .= ',';
 }
 $names .= $value;
 $values .= '?';
 }
 return "INSERT INTO $table ($names) VALUES ($values)";
 break;
 case DB_AUTOQUERY_UPDATE:
 $set = '';
 while (list(, $value) = each($table_fields)) {
 if ($first) {
 $first = false;
 } else {
 $set .= ',';
 }
 $set .= "$value = ?";
 }
 $sql = "UPDATE $table SET $set";
 if ($where) {
 $sql .= " WHERE $sql";
 }
 return $sql;
 break;
 default:
 $this->raiseError(DB_ERROR_SYNTAX);
 }
 }

 
 
 















 function execute($stmt, $data = false)
 {
 $realquery = $this->executeEmulateQuery($stmt, $data);
 if (DB::isError($realquery)) {
 return $realquery;
 }
 $result = $this->simpleQuery($realquery);
 if (DB::isError($result) || $result === DB_OK) {
 return $result;
 } else {
 return new DB_result($this, $result);
 }
 }

 
 

 













 function executeEmulateQuery($stmt, $data = false)
 {
 $p = &$this->prepare_tokens;

 if (!isset($this->prepare_tokens[$stmt]) ||
 !is_array($this->prepare_tokens[$stmt]) ||
 !sizeof($this->prepare_tokens[$stmt]))
 {
 return $this->raiseError(DB_ERROR_INVALID);
 }

 $qq = &$this->prepare_tokens[$stmt];
 $qp = sizeof($qq) - 1;

 if ((!$data && $qp > 0) ||
 (!is_array($data) && $qp > 1) ||
 (is_array($data) && $qp > sizeof($data)))
 {
 $this->last_query = $this->prepared_queries[$stmt];
 return $this->raiseError(DB_ERROR_NEED_MORE_DATA);
 }

 $realquery = $qq[0];
 for ($i = 0; $i < $qp; $i++) {
 $type = $this->prepare_types[$stmt][$i];
 if ($type == DB_PARAM_OPAQUE) {
 if (is_array($data)) {
 $fp = fopen($data[$i], 'r');
 } else {
 $fp = fopen($data, 'r');
 }

 $pdata = '';

 if ($fp) {
 while (($buf = fread($fp, 4096)) != false) {
 $pdata .= $buf;
 }
 }
 } else {
 if (is_array($data)) {
 $pdata = &$data[$i];
 } else {
 $pdata = &$data;
 }
 }

 $realquery .= ($type != DB_PARAM_MISC) ? $this->quote($pdata) : $pdata;
 $realquery .= $qq[$i + 1];
 }

 return $realquery;
 }

 
 

 

















 function executeMultiple( $stmt, &$data )
 {
 for($i = 0; $i < sizeof( $data ); $i++) {
 $res = $this->execute($stmt, $data[$i]);
 if (DB::isError($res)) {
 return $res;
 }
 }
 return DB_OK;
 }

 
 

 










 function modifyQuery($query) {
 return $query;
 }

 
 
 











 function modifyLimitQuery($query, $from, $count)
 {
 return $query;
 }

 
 

 














 function &query($query, $params = array()) {
 if (sizeof($params) > 0) {
 $sth = $this->prepare($query);
 if (DB::isError($sth)) {
 return $sth;
 }
 return $this->execute($sth, $params);
 } else {
 $result = $this->simpleQuery($query);
 if (DB::isError($result) || $result === DB_OK) {
 return $result;
 } else {
 return new DB_result($this, $result);
 }
 }
 }

 
 
 











 function &limitQuery($query, $from, $count)
 {
 $query = $this->modifyLimitQuery($query, $from, $count);
 $result = $this->simpleQuery($query);
 if (DB::isError($result) || $result === DB_OK) {
 return $result;
 } else {
 $options['limit_from'] = $from;
 $options['limit_count'] = $count;
 return new DB_result($this, $result, $options);
 }
 }

 
 

 













 function &getOne($query, $params = array())
 {
 settype($params, "array");
 if (sizeof($params) > 0) {
 $sth = $this->prepare($query);
 if (DB::isError($sth)) {
 return $sth;
 }
 $res = $this->execute($sth, $params);
 } else {
 $res = $this->query($query);
 }

 if (DB::isError($res)) {
 return $res;
 }

 $err = $res->fetchInto($row, DB_FETCHMODE_ORDERED);
 if ($err !== DB_OK) {
 return $err;
 }

 $res->free();

 if (isset($sth)) {
 $this->freeResult($sth);
 }

 return $row[0];
 }

 
 

 












 function &getRow($query,
 $params = null,
 $fetchmode = DB_FETCHMODE_DEFAULT)
 {
 
 
 if (!is_array($params)) {
 if (is_array($fetchmode)) {
 $tmp = $params;
 $params = $fetchmode;
 $fetchmode = $tmp;
 } elseif ($params !== null) {
 $fetchmode = $params;
 $params = null;
 }
 }
 $params = (empty($params)) ? array() : $params;
 $fetchmode = (empty($fetchmode)) ? DB_FETCHMODE_DEFAULT : $fetchmode;
 settype($params, 'array');
 if (sizeof($params) > 0) {
 $sth = $this->prepare($query);
 if (DB::isError($sth)) {
 return $sth;
 }
 $res = $this->execute($sth, $params);
 } else {
 $res = $this->query($query);
 }

 if (DB::isError($res)) {
 return $res;
 }

 $err = $res->fetchInto($row, $fetchmode);

 if ($err !== DB_OK) {
 return $err;
 }
 $res->free();

 if (isset($sth)) {
 $this->freeResult($sth);
 }

 return $row;
 }

 
 

 
















 function &getCol($query, $col = 0, $params = array())
 {
 settype($params, "array");
 if (sizeof($params) > 0) {
 $sth = $this->prepare($query);

 if (DB::isError($sth)) {
 return $sth;
 }

 $res = $this->execute($sth, $params);
 } else {
 $res = $this->query($query);
 }

 if (DB::isError($res)) {
 return $res;
 }

 $fetchmode = is_int($col) ? DB_FETCHMODE_ORDERED : DB_FETCHMODE_ASSOC;
 $ret = array();

 while (is_array($row = $res->fetchRow($fetchmode))) {
 $ret[] = $row[$col];
 }
 if (DB::isError($row)) {
 $ret = $row;
 }
 $res->free();

 if (isset($sth)) {
 $this->freeResult($sth);
 }

 return $ret;
 }

 
 

 









































































 function &getAssoc($query, $force_array = false, $params = array(),
 $fetchmode = DB_FETCHMODE_ORDERED, $group = false)
 {
 settype($params, "array");
 if (sizeof($params) > 0) {
 $sth = $this->prepare($query);

 if (DB::isError($sth)) {
 return $sth;
 }

 $res = $this->execute($sth, $params);
 } else {
 $res = $this->query($query);
 }

 if (DB::isError($res)) {
 return $res;
 }

 $cols = $res->numCols();

 if ($cols < 2) {
 return $this->raiseError(DB_ERROR_TRUNCATED);
 }

 $results = array();

 if ($cols > 2 || $force_array) {
 
 
 if ($fetchmode == DB_FETCHMODE_ASSOC) {
 while (is_array($row = $res->fetchRow(DB_FETCHMODE_ASSOC))) {
 reset($row);
 $key = current($row);
 unset($row[key($row)]);
 if ($group) {
 $results[$key][] = $row;
 } else {
 $results[$key] = $row;
 }
 }
 } else {
 while (is_array($row = $res->fetchRow(DB_FETCHMODE_ORDERED))) {
 
 
 $key = array_shift($row);
 if ($group) {
 $results[$key][] = $row;
 } else {
 $results[$key] = $row;
 }
 }
 }
 if (DB::isError($row)) {
 $results = $row;
 }
 } else {
 
 while (is_array($row = $res->fetchRow(DB_FETCHMODE_ORDERED))) {
 if ($group) {
 $results[$row[0]][] = $row[1];
 } else {
 $results[$row[0]] = $row[1];
 }
 }
 if (DB::isError($row)) {
 $results = $row;
 }
 }

 $res->free();

 if (isset($sth)) {
 $this->freeResult($sth);
 }

 return $results;
 }

 
 

 












 function &getAll($query,
 $params = null,
 $fetchmode = DB_FETCHMODE_DEFAULT)
 {
 
 
 if (!is_array($params)) {
 if (is_array($fetchmode)) {
 $tmp = $params;
 $params = $fetchmode;
 $fetchmode = $tmp;
 } elseif ($params !== null) {
 $fetchmode = $params;
 $params = null;
 }
 }
 $params = (empty($params)) ? array() : $params;
 $fetchmode = (empty($fetchmode)) ? DB_FETCHMODE_DEFAULT : $fetchmode;
 settype($params, "array");
 if (sizeof($params) > 0) {
 $sth = $this->prepare($query);

 if (DB::isError($sth)) {
 return $sth;
 }

 $res = $this->execute($sth, $params);
 } else {
 $res = $this->query($query);
 }

 if (DB::isError($res)) {
 return $res;
 }

 $results = array();
 $this->pushErrorHandling(PEAR_ERROR_RETURN);
 while (DB_OK === $res->fetchInto($row, $fetchmode)) {
 if ($fetchmode & DB_FETCHMODE_FLIPPED) {
 foreach ($row as $key => $val) {
 $results[$key][] = $val;
 }
 } else {
 $results[] = $row;
 }
 }
 $this->popErrorHandling();

 $res->free();

 if (isset($sth)) {
 $this->freeResult($sth);
 }
 if (DB::isError($row)) {
 return $this->raiseError($row);
 }
 return $results;
 }

 
 
 







 function autoCommit($onoff=false)
 {
 return $this->raiseError(DB_ERROR_NOT_CAPABLE);
 }

 
 
 






 function commit()
 {
 return $this->raiseError(DB_ERROR_NOT_CAPABLE);
 }

 
 
 






 function rollback()
 {
 return $this->raiseError(DB_ERROR_NOT_CAPABLE);
 }

 
 
 








 function numRows($result)
 {
 return $this->raiseError(DB_ERROR_NOT_CAPABLE);
 }

 
 
 






 function affectedRows()
 {
 return $this->raiseError(DB_ERROR_NOT_CAPABLE);
 }

 
 
 






 function errorNative()
 {
 return $this->raiseError(DB_ERROR_NOT_CAPABLE);
 }

 
 
 









 function nextId($seq_name, $ondemand = true)
 {
 return $this->raiseError(DB_ERROR_NOT_CAPABLE);
 }

 
 
 








 function createSequence($seq_name)
 {
 return $this->raiseError(DB_ERROR_NOT_CAPABLE);
 }

 
 
 








 function dropSequence($seq_name)
 {
 return $this->raiseError(DB_ERROR_NOT_CAPABLE);
 }

 
 
 









 function tableInfo($result, $mode = null)
 {
 return $this->raiseError(DB_ERROR_NOT_CAPABLE);
 }

 
 
 


 function getTables()
 {
 return $this->getListOf('tables');
 }

 
 
 










 function getListOf($type)
 {
 $sql = $this->getSpecialQuery($type);
 if ($sql === null) { 
 return $this->raiseError(DB_ERROR_UNSUPPORTED);
 } elseif (is_int($sql) || DB::isError($sql)) { 
 return $this->raiseError($sql);
 } elseif (is_array($sql)) { 
 return $sql;
 }
 return $this->getCol($sql); 
 }
 
 

 function getSequenceName($sqn)
 {
 return sprintf($this->getOption("seqname_format"),
 preg_replace('/[^a-z0-9_]/i', '_', $sqn));
 }

 
}


if (!function_exists('array_change_key_case')) {
 define('CASE_UPPER', 1);
 define('CASE_LOWER', 0);
 function &array_change_key_case(&$array, $case) {
 $casefunc = ($case == CASE_LOWER) ? 'strtolower' : 'strtoupper';
 $ret = array();
 foreach ($array as $key => $value) {
 $ret[$casefunc($key)] = $value;
 }
 return $ret;
 }
}

?>